﻿using System;
using System.IO;
using System.Threading;
using System.Configuration;
using System.Web;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using StammbuchService;
using System.Net;

namespace DropBoxClient
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            Properties.Settings set = new DropBoxClient.Properties.Settings();

            Password = set.Password;
        }

        private String Password;

        delegate void SetTextCallback(String value);
        delegate void SetStatusCallback(String value);
        delegate void SetStatusProgressbarCallback(Int32 value);

        private void SetURLText(String value)
        {
            try
            {
                if (this.URL.InvokeRequired)
                {
                    SetTextCallback d = new SetTextCallback(SetURLText);
                    this.Invoke(d, new object[] { value });
                }
                else
                {
                    this.URL.Text = value;
                }
            }
            catch (Exception e)
            {
            }
        }

        private void SetStatusText(String value)
        {
            try
            {
                if (this.URL.InvokeRequired)
                {
                    SetStatusCallback d = new SetStatusCallback(SetStatusText);
                    this.Invoke(d, new object[] { value });
                }
                else
                {
                    this.StatusBarLabel.Text = value;
                }
            }
            catch (Exception e)
            {
            }
        }

        private void SetStatusProgressBar(Int32 value)
        {
            try
            {
                if (this.URL.InvokeRequired)
                {
                    SetStatusProgressbarCallback d = new SetStatusProgressbarCallback(SetStatusProgressBar);
                    this.Invoke(d, new object[] { value });
                }
                else
                {
                    this.StatusProgressBar.Value = value;
                }
            }
            catch (Exception e)
            {
            }
        }

        private String[] FilesToUpload;
        private bool currentlyWorking = false;

        public void Upload()
        {
            currentlyWorking = true;
            DropBox.DropboxSoapClient Client = new DropBoxClient.DropBox.DropboxSoapClient();

            if (!File.Exists(FilesToUpload[0]))
            {
                currentlyWorking = false;
                MessageBox.Show("File does not exist");
                return;
            }
            foreach (String fullfilename in FilesToUpload)
            {
                String filename = fullfilename.Remove(0, fullfilename.LastIndexOf("\\") + 1);

                try
                {
                    String blah = Client.CheckFileName(filename, Password);
                }
                catch (Exception)
                {
                    SetStatusText("Webservice isn't talking to us at the moment - maybe overloaded - try again later.");
                    return;
                }


                Thumbnailer thmb = new Thumbnailer();

                Bitmap Input;
                Bitmap Output;

                if (filename.ToUpper().EndsWith("JPG"))
                {
                    Input = thmb.LoadOriginalImage(fullfilename);
                    Output = thmb.CreateNewImage(Input, 320, false);
                    byte[] Data = thmb.GetImageBytes(Output);

                    long numBytes = Data.Length;
                    double dLen = Convert.ToDouble(Data.Length / 1000000);
                    // default is 4 Megabyte maximum

                    try
                    {
                        if (dLen > 4)
                        {
                            #region Upload bigger 4 Mbytes
                            // check if the filename is okay...
                            String InternalFilename = Client.CheckFileName(filename, Password);

                            String URL = "";
                            long TransferedBytes = 0;
                            long RemainingBytes = numBytes;
                            Int32 ReadNextBytes = 0;
                            long segmentID = 0;
                            Int32 ChunkSize = 1000000;

                            MemoryStream fStream = new MemoryStream(Data);
                            BinaryReader br = new BinaryReader(fStream);
                            SetStatusText("Uploading " + filename + " as " + InternalFilename);

                            Int32 CurrentPercent = 0;

                            while (TransferedBytes != numBytes)
                            {
                                CurrentPercent = (Int32)Math.Ceiling((double)(TransferedBytes / (numBytes / 100)));

                                //SetStatusText("Uploading " + filename + " as " + InternalFilename+ " with "+RemainingBytes+" Bytes remaining...");

                                SetStatusProgressBar(CurrentPercent);
                                segmentID++;

                                if (RemainingBytes >= ChunkSize)
                                {
                                    ReadNextBytes = ChunkSize;
                                    RemainingBytes = RemainingBytes - ChunkSize;
                                }
                                else
                                {
                                    if (RemainingBytes == 0)
                                        break;

                                    ReadNextBytes = (Int32)RemainingBytes;
                                    RemainingBytes = 0;
                                }

                                TransferedBytes = TransferedBytes + ReadNextBytes;
                                byte[] data = br.ReadBytes(ReadNextBytes);

                                bool finished = false;

                                while (!finished)
                                {
                                    try
                                    {
                                        URL = Client.DropFile(data, InternalFilename, true, Password);
                                        finished = true;
                                    }
                                    catch (Exception)
                                    {
                                        Client.Close();
                                        Client = new DropBoxClient.DropBox.DropboxSoapClient();
                                        SetStatusText("An error occured while uploading... Retrying");
                                        finished = false;
                                    }
                                }
                            }
                            br.Close();
                            Clipboard.SetDataObject(URL, true);

                            SetStatusProgressBar(100);

                            SetURLText(URL);
                            SetStatusText("File successfully uploaded...");
                            #endregion
                        }
                        else
                        {
                            #region Upload smaller 4 Mbytes
                            SetStatusText("Uploading " + filename);
                            MemoryStream fStream = new MemoryStream(Data);
                            BinaryReader br = new BinaryReader(fStream);
                            // convert the file to a byte array
                            byte[] data = br.ReadBytes((int)numBytes);
                            SetStatusProgressBar(50);
                            br.Close();
                            // pass the byte array (file) and file name to the web service
                            //string sTmp = srv.UploadFile(data, strFile);
                            String URL = Client.DropFile(data, filename, false, Password);
                            fStream.Close();
                            fStream.Dispose();
                            //MessageBox.Show("File Upload Status: " + sTmp, "File Upload");                
                            Clipboard.SetDataObject(URL, true);
                            SetURLText(URL);
                            SetStatusProgressBar(100);
                            SetStatusText("File successfully uploaded...");
                            #endregion
                        }
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show(e.Message);
                    }
                }

                SetStatusProgressBar(0);
            }
            currentlyWorking = false;
        }

        private void label1_DragDrop(object sender, DragEventArgs e)
        {
            FilesToUpload = (string[])e.Data.GetData(DataFormats.FileDrop, false);

            Thread upload_thread = new Thread(new ThreadStart(Upload));
            upload_thread.SetApartmentState(ApartmentState.STA);
            upload_thread.Start();
        }

        private void label1_DragEnter(object sender, DragEventArgs e)
        {
            if ((!currentlyWorking) && (e.Data.GetDataPresent(DataFormats.FileDrop)))
                e.Effect = DragDropEffects.All;
            else
                e.Effect = DragDropEffects.None;

        }

        private void ListFiles_Click(object sender, EventArgs e)
        {
            DropBox.DropboxSoapClient Client = new DropBoxClient.DropBox.DropboxSoapClient();

            try
            {
                DropBox.ArrayOfString Files = Client.ListDroppedFiles(Password);

                FileListing.Items.Clear();

                foreach (String item in Files)
                {
                    String Filename = item.Remove(0, item.LastIndexOf("/") + 1);

                    FileListing.Items.Add(new ListBoxItem(Filename, HttpUtility.UrlPathEncode(item)));
                }
            }
            catch (Exception)
            {
                FileListing.Items.Add("Webservice isn't talking to us at the moment - maybe overloaded - try again later.");
            }
        }

        private void FileListing_SelectedValueChanged(object sender, EventArgs e)
        {
            ListBoxItem URLItem = (ListBoxItem)FileListing.SelectedItem;
            Clipboard.SetDataObject(URLItem.URL, true);

            WebClient Client = new WebClient();
            Stream strm = Client.OpenRead(URLItem.URL);

            Thumbnailer thmb = new Thumbnailer();

            pictureBox2.Image = Bitmap.FromStream(strm);

            strm.Close();
        }

        private void deletebutton_Click(object sender, EventArgs e)
        {
            if (FileListing.SelectedItem != null)
            {
                ListBoxItem URLItem = (ListBoxItem)FileListing.SelectedItem;

                DropBox.DropboxSoapClient Client = new DropBoxClient.DropBox.DropboxSoapClient();

                try
                {
                    DropBox.ArrayOfString Files = Client.DeleteFile(URLItem.Filename, Password);
                    FileListing.Items.Clear();
                    foreach (String item in Files)
                    {
                        String Filename = item.Remove(0, item.LastIndexOf("/") + 1);


                        FileListing.Items.Add(new ListBoxItem(Filename, HttpUtility.UrlPathEncode(item)));

                    }
                }
                catch (Exception ee)
                {
                    FileListing.Items.Add(ee.Message);
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.pictureBox1.AllowDrop = true;
        }
    }

    public class ListBoxItem
    {
        public String Filename;
        public String URL;

        public ListBoxItem(String Filen, String URLn)
        {
            Filename = Filen;
            URL = URLn;
        }

        public override string  ToString()
        {
 	         return Filename;
        }
    }
}
